@@ -1,7 +1,7 @@
 /************************************************************************
  * linux.c                                                              *
  *                                                                      *
- * Copyright (C) 2002 - 2019 Joshua Birnbaum <engineer@noorg.org>.      *
+ * Copyright (C) 2002 - 2020 Joshua Birnbaum <engineer@noorg.org>.      *
  * All Rights Reserved.                                                 *
  *                                                                      *
  * This program is free software; you can redistribute it and/or modify *
@@ -31,7 +31,7 @@
  * Project ifchk -- Host based network interface promiscuous   *
  *                  mode detection and handling.               *
  *                                                             *
- * Version:         1.1.1                                      *
+ * Version:         1.1.2                                      *
  *                                                             *
  * ----------------------------------------------------------- *
  *                                                             *
@@ -71,29 +71,31 @@ int bFlag = FALSE;
 
 int ifStat( void )
 {
-    char mntPath[SYSFS_PATH_MAX] = "";		/* Buffer for sysfs filesystem mount point pathname. */
-    const char *className = NULL;		/* sysfs class to open (e.g., "net"). */
-    struct sysfs_class *class;			/* sysfs class structure. */
-    struct dlist *classDev;			/* sysfs class device list. */
-    char pathName[SYSFS_PATH_MAX] = "";		/* Pathname of sysfs object to query. */
-    char *bondPtr = NULL;			/* Pathname of bond interface sysfs object to query. */
-    char slaveFile[BONDBUFSZ] = "";		/* Buffer for sysfs filesystem bonding interface slaves. */
-    FILE *bondslavefd = NULL;			/* File pointer that points to bonding interface slaves. */ 
-    char buf[BONDBUFSZ] = "";			/* Buffer for sysfs filesystem bonding interface slaves. */
-    char *nlPtr = NULL;				/* Ptr for trailing newline processing. */
-    int ifCount = 0;				/* Interface count. */
-    struct sysfs_attribute *sysAttr;		/* sysfs attribute to open (e.g., /sys/class/net/eth0/flags). */
-    char *endPtr = NULL;			/* For strtol(). */
-    int i = 0;					/* For ifMetric(). */
-    long j = 0;					/* Interface flags mask. */
-    int sockfd = 0;				/* Socket descriptor for ioctl(). */
-    struct ifreq ifr;				/* Per-interface characteristics. */
-    struct ifList *head = NULL;			/* Linked list of interface \ */
-    struct ifList *tail = NULL;			/* name/unit number pairings. \ */
-    struct ifList *new = NULL;			/* Constructed locally & passed \ */
-    struct ifList *cur = NULL;			/* to ifMetric() for processing \ */
-    struct ifList *freeList = NULL;		/* & output. */
-    struct fsObj slaveObject;			/* File attribute checking policy. */ 
+    char mntPath[SYSFS_PATH_MAX] = "";      /* Buffer for sysfs filesystem mount point pathname. */
+    const char *className = NULL;           /* sysfs class to open (e.g., "net"). */
+    struct sysfs_class *class;              /* sysfs class structure. */
+    struct dlist *classDev;                 /* sysfs class device list. */
+    char pathName[SYSFS_PATH_MAX] = "";     /* Pathname of sysfs object to query. */
+    char *bondPtr = NULL;                   /* Pathname of bond interface sysfs object to query. */
+    char slaveFile[BONDBUFSZ] = "";         /* Pathname of sysfs filesystem bonding interface slaves. */
+    char actSlave[BONDBUFSZ] = "";          /* Pathname of sysfs filesystem active slave interface. */
+    FILE *bondslavefd = NULL;               /* File pointer that points to bonding interface slaves. */
+    char buf[BONDBUFSZ] = "";               /* Buffer for sysfs filesystem bonding interface slaves. */
+    char slaveBuf[BONDBUFSZ] = "";          /* Buffer for sysfs filesystem binding active slave interface. */
+    char *nlPtr = NULL;                     /* Ptr for trailing newline processing. */
+    int ifCount = 0;                        /* Interface count. */
+    struct sysfs_attribute *sysAttr;        /* sysfs attribute to open (e.g., /sys/class/net/eth0/flags). */
+    char *endPtr = NULL;                    /* For strtol(). */
+    int i = 0;                              /* For ifMetric(). */
+    long j = 0;                             /* Interface flags mask. */
+    int sockfd = 0;                         /* Socket descriptor for ioctl(). */
+    struct ifreq ifr;                       /* Per-interface characteristics. */
+    struct ifList *head = NULL;             /* Linked list of interface \ */
+    struct ifList *tail = NULL;             /* name/unit number pairings. \ */
+    struct ifList *new = NULL;              /* Constructed locally & passed \ */
+    struct ifList *cur = NULL;              /* to ifMetric() for processing \ */
+    struct ifList *freeList = NULL;         /* & output. */
+    struct fsObj slaveObject;               /* File attribute checking policy. */
 
     /*
      * Reset all statFile() file checks.
@@ -138,7 +140,7 @@ int ifStat( void )
     if( ( class = sysfs_open_class( className ) ) == NULL )
     {
         errHandler( "ERROR: sysfs_open_class(): could not open sysfs class \"%s\": %s\n",
-	                                                     className, strerror( errno ) );
+                                                             className, strerror( errno ) );
         return (-1);
      }
 
@@ -162,7 +164,7 @@ int ifStat( void )
     {
         printf( "interfaces: %d\n", ifCount );
 
-	if( ( writeLog( LOG_INFO, "interfaces: %d\n", ifCount ) ) != 0 )
+        if( ( writeLog( LOG_INFO, "interfaces: %d\n", ifCount ) ) != 0 )
         {
             return (-1);
          }
@@ -186,7 +188,7 @@ int ifStat( void )
          */
         memset( pathName, '\000', sizeof( pathName ) );
         strncpy( pathName, class -> path, sizeof( pathName ) );
-	pathName[SYSFS_PATH_MAX - 1] = '\000';
+        pathName[SYSFS_PATH_MAX - 1] = '\000';
         strncat( pathName, "/", ( sizeof( pathName ) - strlen( pathName ) ) - 1 );
         strncat( pathName, "flags", ( sizeof( pathName ) - strlen( pathName ) ) - 1 );
 
@@ -198,26 +200,23 @@ int ifStat( void )
             bFlag = TRUE;
 
             /*
-	     * Build a pathname to the current bonding interface's slave sysfs
-	     * file (e.g., /sys/devices/virtual/net/bond0/bonding/slaves).
-	     * This file describes the slave interfaces (e.g., em1, em2) for
-	     * which the bond is master.
-	     */
+             * Build a full path to the bonding interface slave sysfs file.
+             */
             strncpy( slaveFile, "/sys/devices/virtual/net/", sizeof( slaveFile ) );
             strncat( slaveFile, class -> name, sizeof( slaveFile ) );
             strncat( slaveFile, "/bonding/slaves", sizeof( slaveFile ) );
 
-	    /*
-             * Prepare to securely open the current bonding slave's sysfs file.
-	     */
+            /*
+             * Prepare to securely open the slave sysfs file.
+             */
             if( ( bondslavefd = statFile( slaveFile, slaveObject ) ) == NULL )
             {
                 return (-1);
              }
 
             /*
-	     * Once opened, read the file's contents (e.g., "em1 em2").
-	     */
+             * Once opened, read the file's contents (e.g., "em1 em2").
+             */
             while( ( fgets( buf, sizeof( buf ), bondslavefd ) ) != NULL )
             {
                 if( ( nlPtr = strchr( buf, '\n' ) ) )
@@ -227,18 +226,53 @@ int ifStat( void )
              }
 
             /*
-	     * Once read, close the slave sysfs file.
-	     */
+             * Once read, close the slave sysfs file.
+             */
+            if( fclose( bondslavefd ) != 0 )
+            {
+                errHandler( "ERROR: fclose(): could not close %s: %s\n", slaveFile, strerror( errno ) );
+                return (-1);
+             }
+
+            /*
+             * Build a full path to the active slave interface sysfs file.
+             */
+            strncpy( actSlave, "/sys/devices/virtual/net/", sizeof( actSlave ) );
+            strncat( actSlave, class -> name, sizeof( actSlave ) );
+            strncat( actSlave, "/bonding/active_slave", sizeof( actSlave ) );
+
+            /*
+             * Prepare to securely open the active slave sysfs file.
+             */
+            if( ( bondslavefd = statFile( actSlave, slaveObject ) ) == NULL )
+            {
+                return (-1);
+             }
+
+            /*
+             * Once opened, read the file's contents (e.g., "em1").
+             */
+            while( ( fgets( slaveBuf, sizeof( slaveBuf ), bondslavefd ) ) != NULL )
+            {
+                if( ( nlPtr = strchr( slaveBuf, '\n' ) ) )
+                {
+                    *nlPtr = '\000';
+                 }
+             }
+
+            /*
+             * Once read, close the slave sysfs file.
+             */
             if( fclose( bondslavefd ) != 0 )
             {
                 errHandler( "ERROR: fclose(): could not close %s: %s\n", slaveFile, strerror( errno ) );
                 return (-1);
              }
          }
-	 
+
         /*
-	 * Open the current interface's sysfs flags file.
-	 */
+         * Open the current interface's sysfs flags file.
+         */
         if( ( sysAttr = sysfs_open_attribute( pathName ) ) == NULL )
         {
             errHandler( "ERROR: sysfs_open_attribute(): could not get configured interface flags: %s\n",
@@ -246,9 +280,9 @@ int ifStat( void )
             return (-1);
          }
 
-	/*
-	 * Read the current interface's sysfs flags file.
-	 */
+        /*
+         * Read the current interface's sysfs flags file.
+         */
         if( sysfs_read_attribute( sysAttr ) == -1 )
         {
             errHandler( "ERROR: sysfs_read_attribute(): could not get configured interface flags: %s\n",
@@ -258,19 +292,19 @@ int ifStat( void )
 
         j = strtol( sysAttr -> value, &endPtr, 16 );
 
-	/*
-	 * The current flags value contains illegal characters.
-	 */
+        /*
+         * The current flags value contains illegal characters.
+         */
         if( ( sysAttr -> value == endPtr ) || ( *endPtr != '\n' ) )
         {
             errHandler( "ERROR: strtol(): flags conversion for interface %s failed\n", class -> name );
-	    return (-1);
+            return (-1);
          }
 
         /*
-	 * Interface is promiscuous - print & log state.
-	 */
-	if( ( j & IFF_PROMISC ) && ( nullFlag == TRUE ) )
+         * Interface is promiscuous - print & log state.
+         */
+        if( ( j & IFF_PROMISC ) && ( nullFlag == TRUE ) )
         {
             if( bFlag == TRUE )
             {
@@ -294,9 +328,9 @@ int ifStat( void )
          }
 
         /*
-	 * Interface is promiscuous - disable it, print & log state.
-	 */
-	else if( ( j & IFF_PROMISC ) && ( dFlag == TRUE ) )
+         * Interface is promiscuous - disable it, print & log state.
+         */
+        else if( ( j & IFF_PROMISC ) && ( dFlag == TRUE ) )
         {
             if( ( sockfd = socket( AF_INET, SOCK_DGRAM, 0 ) ) == -1 )
             {
@@ -307,7 +341,7 @@ int ifStat( void )
 
             strncpy( ifr.ifr_name, class -> name, sizeof( ifr.ifr_name ) );
 
-	    ifr.ifr_name[IFNAMSIZ - 1] = '\000';
+            ifr.ifr_name[IFNAMSIZ - 1] = '\000';
 
             if( ioctl( sockfd, SIOCGIFFLAGS, &ifr ) < 0 )
             {
@@ -352,10 +386,10 @@ int ifStat( void )
              }
          }
 
-	/*
-	 * Construct linked list of interface names for ifMetric().
-	 */
-	else if( iFlag == TRUE )
+        /*
+         * Construct linked list of interface names for ifMetric().
+         */
+        else if( iFlag == TRUE )
         {
             if( ( new = malloc( sizeof( struct ifList ) ) ) == NULL )
             {
@@ -363,9 +397,9 @@ int ifStat( void )
                 return (-1);
              }
 
-	    /*
-	     * Create a new node.
-	     */
+            /*
+             * Create a new node.
+             */
             memset( new, '\000', sizeof( new ) );
             strncpy( new -> data, class -> name, sizeof( new -> data ) );
             new -> data[IFNAMSIZ - 1] = '\000';
@@ -389,10 +423,10 @@ int ifStat( void )
 
             i++;
 
-	    /*
-	     * When the list is complete, call ifMetric(),
-	     * passing it a pointer to the head node.
-	     */
+            /*
+             * When the list is complete, call ifMetric(),
+             * passing it a pointer to the head node.
+             */
             if( i == ifCount )
             {
                 if( ifMetric( head ) != 0 )
@@ -400,9 +434,9 @@ int ifStat( void )
                     return (-1);
                  }
 
-		/*
-		 * Deallocate list memory after use.
-		 */
+                /*
+                 * Deallocate list memory after use.
+                 */
                 for( cur = head; cur != NULL; cur = freeList )
                 {
                     freeList = cur -> next;
@@ -411,18 +445,17 @@ int ifStat( void )
              }
          }
 
-	/*
-	 * Interface is up - print & log state.
-	 */
+        /*
+         * Interface is up - print & log state.
+         */
         else if( j & IFF_UP )
         {
             if( bFlag == TRUE )
             {
-                /* printf( "%6s: normal\n", class -> name ); */
-                printf( "%6s: normal (slave: %s)\n", class -> name, buf );
+                printf( "%6s: normal <slave: %s, actv, %s>\n", class -> name, buf, slaveBuf );
 
-                /* if( ( writeLog( LOG_INFO, "%4s: normal\n", class -> name ) ) != 0 ) */
-                if( ( writeLog( LOG_INFO, "%4s: normal (slave: %s)\n", class -> name, buf ) ) != 0 )
+                if( ( writeLog( LOG_INFO, "%4s: normal <slave: %s, actv: %s>\n",
+                                               class -> name, buf, slaveBuf ) ) != 0 )
                 {
                     return (-1);
                  }
@@ -439,16 +472,15 @@ int ifStat( void )
              }
          }
 
-	/*
-	 * Interface is down - print & log state.
-	 */
+        /*
+         * Interface is down - print & log state.
+         */
         else
         {
             if( bFlag == TRUE )
             {
-                printf( "%6s: *down* (slave: %s)\n", class -> name, buf );
-
-                if( ( writeLog( LOG_INFO, "%4s: *down* (slave: %s)\n", class -> name, buf ) ) != 0 )
+                if( ( writeLog( LOG_INFO, "%4s: *down* <slave: %s, actv: %s>\n",
+                                               class -> name, buf, slaveBuf ) ) != 0 )
                 {
                     return (-1);
                  }
@@ -466,8 +498,8 @@ int ifStat( void )
          }
 
         /*
-	 * Close the current interface's sysfs flags file.
-	 */
+         * Close the current interface's sysfs flags file.
+         */
         sysfs_close_attribute( sysAttr );
 
         bFlag = FALSE;
@@ -496,19 +528,19 @@ int ifStat( void )
 
 int ifMetric( struct ifList *list )
 {
-    const char *className = NULL;		/* sysfs class to open (e.g., "net"). */
-    struct sysfs_class *class;			/* sysfs class structure. */
-    struct dlist *classDev;			/* sysfs class device list. */
-    char pathName[SYSFS_PATH_MAX] = "";		/* Pathname of sysfs object to query. */
-    char *classPath = NULL;			/* Pathname of sysfs class (e.g., /sys/class/net). */
-    struct sysfs_class_device *devAttrs;	/* Describes class device (e.g., eth0) attributes. */
-    struct sysfs_attribute *attr;		/* sysfs attribute to open (e.g., /sys/class/net/eth0/ifindex). */
-    char *ifIndex = NULL;			/* Interface index. */
-    char *rxPackets = NULL;			/* Received packet count. */
-    char *txPackets = NULL;			/* Transmitted packet count. */
-    char ifMetricBuf[METRICBUFSZ];		/* Buffer for interface metrics data. */
-    char *lfPtr = NULL;				/* Ptr for trailing new line processing. */
-    struct ifList *cur = NULL;			/* Current node in linked list. */
+    const char *className = NULL;           /* sysfs class to open (e.g., "net"). */
+    struct sysfs_class *class;              /* sysfs class structure. */
+    struct dlist *classDev;                 /* sysfs class device list. */
+    char pathName[SYSFS_PATH_MAX] = "";     /* Pathname of sysfs object to query. */
+    char *classPath = NULL;                 /* Pathname of sysfs class (e.g., /sys/class/net). */
+    struct sysfs_class_device *devAttrs;    /* Describes class device (e.g., eth0) attributes. */
+    struct sysfs_attribute *attr;           /* sysfs attribute to open (e.g., /sys/class/net/eth0/ifindex). */
+    char *ifIndex = NULL;                   /* Interface index. */
+    char *rxPackets = NULL;                 /* Received packet count. */
+    char *txPackets = NULL;                 /* Transmitted packet count. */
+    char ifMetricBuf[METRICBUFSZ];          /* Buffer for interface metrics data. */
+    char *lfPtr = NULL;                     /* Ptr for trailing new line processing. */
+    struct ifList *cur = NULL;              /* Current node in linked list. */
 
     cur = list;
 
@@ -534,7 +566,7 @@ int ifMetric( struct ifList *list )
     {
         errHandler( "ERROR: sysfs_open_class(): could not open sysfs class \"%s\": %s\n",
                                                              className, strerror( errno ) );
-	return (-1);
+        return (-1);
      }
 
     /*
@@ -543,7 +575,7 @@ int ifMetric( struct ifList *list )
      */
     if( ( classDev = sysfs_get_class_devices( class ) ) == NULL )
     {
-	errHandler( "ERROR: sysfs_get_class_devices(): could not get class device list: %s\n",
+        errHandler( "ERROR: sysfs_get_class_devices(): could not get class device list: %s\n",
                                                                              strerror( errno ) );
         return (-1);
      }
@@ -565,7 +597,7 @@ int ifMetric( struct ifList *list )
         return (-1);
      }
 
-    if( ( writeLog( LOG_INFO, "Name    Index   RX-OK           TX-OK\n" ) ) != 0 )
+    if( ( writeLog( LOG_INFO, "Name           Index   RX-OK           TX-OK\n" ) ) != 0 )
     {
         return (-1);
      }
@@ -579,146 +611,146 @@ int ifMetric( struct ifList *list )
         if( ( strstr( class -> name, cur -> data ) ) == NULL )
         {
             errHandler( "%s", "ERROR: current metrics do not describe current interface\n" );
-	    return (-1);
-         }
-
-	/*
-	 * Build a pathname to the current interface's
-	 * sysfs directory (e.g., /sys/class/net/eth0 ).
-	 */
-        memset( pathName, '\000', sizeof( pathName ) );
-        strncpy( pathName, classPath, sizeof( pathName ) );
-	pathName[SYSFS_PATH_MAX - 1] = '\000';
-        strncat( pathName, "/", ( sizeof( pathName ) - strlen( pathName ) ) - 1 );
-        strncat( pathName, cur -> data, ( sizeof( pathName ) - strlen( pathName ) ) - 1 );
-
-	/*
-	 * Open the directory.
-	 */
-	if( ( devAttrs = sysfs_open_class_device_path( pathName ) ) == NULL )
-        {
-            errHandler( "ERROR: sysfs_open_class_device_path(): could not open sysfs class device: %s\n",
-                                                                                        strerror( errno ) );
-	    return (-1);
-         }
-
-	/*
-	 * Get the index for the current interface
-	 * (e.g., /sys/class/net/eth0/ifindex).
-	 */
-	ifIndex = "ifindex";
-
-	if( ( attr = sysfs_get_classdev_attr( devAttrs, ifIndex ) ) == NULL )
-        {
-            errHandler( "ERROR: sysfs_get_classdev_attr(): could not get sysfs class device attribute: %s\n",
-                                                                                            strerror( errno ) );
-	    return (-1);
-         }
-
-	ifIndex = attr -> value;
-
-	if( ( lfPtr = strchr( ifIndex, '\n' ) ) == NULL )
-        {
-            errHandler( "%s", "ERROR: strchr(): could not process interface metrics data: no newline terminator\n" );
-	    return (-1);
-         }
-
-	else
-        {
-            *lfPtr = '\000';
-         }
-
-	/*
-	 * Append "/statistics" to pathName in preparation for
-	 * reading packets received (rx_packets) and transmitted
-	 * (tx_packets) on the current interface.
-	 */
-        strncat( pathName, "/", ( sizeof( pathName ) - strlen( pathName ) ) - 1 );
-        strncat( pathName, "statistics", ( sizeof( pathName ) - strlen( pathName ) ) - 1 );
-
-	/*
-	 * Open the statistics directory.
-	 */
-	if( ( devAttrs = sysfs_open_class_device_path( pathName ) ) == NULL )
-        {
-            errHandler( "ERROR: sysfs_open_class_device_path(): could not open sysfs class device: %s\n",
-                                                                                        strerror( errno ) );
-	    return (-1);
-         }
-
-	/*
-	 * Get received packet count for the current interface
-	 * (e.g., /sys/class/net/eth0/statistics/rx_packets).
-	 */
-	rxPackets = "rx_packets";
-
-	if( ( attr = sysfs_get_classdev_attr( devAttrs, rxPackets ) ) == NULL )
-        {
-            errHandler( "ERROR: sysfs_get_classdev_attr(): could not get sysfs class device attribute: %s\n",
-                                                                                            strerror( errno ) );
-	    return (-1);
-         }
-
-	rxPackets = attr -> value;
-
-	if( ( lfPtr = strchr( rxPackets, '\n' ) ) == NULL )
-        {
-            errHandler( "%s", "ERROR: strchr(): could not process interface metrics data: no newline terminator\n" );
-	    return (-1);
-         }
-
-	else
-        {
-            *lfPtr = '\000';
-         }
-
-	/*
-	 * Get transmitted packet count for the current interface
-	 * (e.g., /sys/class/net/eth0/statistics/tx_packets).
-	 */
-	txPackets = "tx_packets";
-
-	if( ( attr = sysfs_get_classdev_attr( devAttrs, txPackets ) ) == NULL )
-	{
-            errHandler( "ERROR: sysfs_get_classdev_attr(): could not get sysfs class device attribute: %s\n",
-                                                                                            strerror( errno ) );
-	    return (-1);
-	 }
-
-	txPackets = attr -> value;
-
-	if( ( lfPtr = strchr( txPackets, '\n' ) ) == NULL )
-	{
-            errHandler( "%s", "ERROR: strchr(): could not process interface metrics data: no newline terminator\n" );
-	    return (-1);
-	 }
-
-	else
-        {
-            *lfPtr = '\000';
-         }
-
-        memset( ifMetricBuf, '\000', sizeof( ifMetricBuf) );
-
-	/*
-	 * Output one line of metrics data for the current interface.
-	 */
-	if( snprintf( ifMetricBuf, sizeof( ifMetricBuf ), "%-7s %-7s %-15s %-13s", cur -> data,
-                                                            ifIndex, rxPackets, txPackets ) < 1 )
-        {
-	    errHandler( "ERROR: snprintf(): could not write to character buffer: %s\n",
-                                                                      strerror( errno ) );
-	    return (-1);
-         }
-
-	printf( "%s\n", ifMetricBuf );
-
-	if( ( writeLog( LOG_INFO, "%s\n", ifMetricBuf ) ) != 0 )
-        {
             return (-1);
          }
 
-	if( cur -> next != NULL )
+       /*
+        * Build a pathname to the current interface's
+        * sysfs directory (e.g., /sys/class/net/eth0 ).
+        */
+        memset( pathName, '\000', sizeof( pathName ) );
+        strncpy( pathName, classPath, sizeof( pathName ) );
+        pathName[SYSFS_PATH_MAX - 1] = '\000';
+        strncat( pathName, "/", ( sizeof( pathName ) - strlen( pathName ) ) - 1 );
+        strncat( pathName, cur -> data, ( sizeof( pathName ) - strlen( pathName ) ) - 1 );
+
+        /*
+         * Open the directory.
+         */
+        if( ( devAttrs = sysfs_open_class_device_path( pathName ) ) == NULL )
+        {
+            errHandler( "ERROR: sysfs_open_class_device_path(): could not open sysfs class device: %s\n",
+                                                                                        strerror( errno ) );
+            return (-1);
+         }
+    
+        /*
+         * Get the index for the current interface
+         * (e.g., /sys/class/net/eth0/ifindex).
+         */
+        ifIndex = "ifindex";
+    
+        if( ( attr = sysfs_get_classdev_attr( devAttrs, ifIndex ) ) == NULL )
+        {
+            errHandler( "ERROR: sysfs_get_classdev_attr(): could not get sysfs class device attribute: %s\n",
+                                                                                            strerror( errno ) );
+            return (-1);
+         }
+    
+        ifIndex = attr -> value;
+    
+        if( ( lfPtr = strchr( ifIndex, '\n' ) ) == NULL )
+        {
+            errHandler( "%s", "ERROR: strchr(): could not process interface metrics data: no newline terminator\n" );
+            return (-1);
+         }
+    
+        else
+        {
+            *lfPtr = '\000';
+         }
+    
+        /*
+         * Append "/statistics" to pathName in preparation for
+         * reading packets received (rx_packets) and transmitted
+         * (tx_packets) on the current interface.
+         */
+        strncat( pathName, "/", ( sizeof( pathName ) - strlen( pathName ) ) - 1 );
+        strncat( pathName, "statistics", ( sizeof( pathName ) - strlen( pathName ) ) - 1 );
+    
+        /*
+         * Open the statistics directory.
+         */
+        if( ( devAttrs = sysfs_open_class_device_path( pathName ) ) == NULL )
+        {
+            errHandler( "ERROR: sysfs_open_class_device_path(): could not open sysfs class device: %s\n",
+                                                                                        strerror( errno ) );
+            return (-1);
+         }
+    
+        /*
+         * Get received packet count for the current interface
+         * (e.g., /sys/class/net/eth0/statistics/rx_packets).
+         */
+        rxPackets = "rx_packets";
+    
+        if( ( attr = sysfs_get_classdev_attr( devAttrs, rxPackets ) ) == NULL )
+        {
+            errHandler( "ERROR: sysfs_get_classdev_attr(): could not get sysfs class device attribute: %s\n",
+                                                                                            strerror( errno ) );
+            return (-1);
+         }
+    
+        rxPackets = attr -> value;
+    
+        if( ( lfPtr = strchr( rxPackets, '\n' ) ) == NULL )
+        {
+            errHandler( "%s", "ERROR: strchr(): could not process interface metrics data: no newline terminator\n" );
+            return (-1);
+         }
+    
+        else
+        {
+            *lfPtr = '\000';
+         }
+    
+        /*
+         * Get transmitted packet count for the current interface
+         * (e.g., /sys/class/net/eth0/statistics/tx_packets).
+         */
+        txPackets = "tx_packets";
+    
+        if( ( attr = sysfs_get_classdev_attr( devAttrs, txPackets ) ) == NULL )
+        {
+            errHandler( "ERROR: sysfs_get_classdev_attr(): could not get sysfs class device attribute: %s\n",
+                                                                                            strerror( errno ) );
+            return (-1);
+         }
+    
+        txPackets = attr -> value;
+    
+        if( ( lfPtr = strchr( txPackets, '\n' ) ) == NULL )
+        {
+            errHandler( "%s", "ERROR: strchr(): could not process interface metrics data: no newline terminator\n" );
+            return (-1);
+         }
+    
+        else
+        {
+            *lfPtr = '\000';
+         }
+    
+        memset( ifMetricBuf, '\000', sizeof( ifMetricBuf) );
+    
+        /*
+         * Output one line of metrics data for the current interface.
+         */
+        if( snprintf( ifMetricBuf, sizeof( ifMetricBuf ), "%-14s %-7s %-15s %-13s", cur -> data,
+                                                                ifIndex, rxPackets, txPackets ) < 1 )
+        {
+            errHandler( "ERROR: snprintf(): could not write to character buffer: %s\n",
+                                                                      strerror( errno ) );
+            return (-1);
+         }
+    
+        printf( "%s\n", ifMetricBuf );
+    
+        if( ( writeLog( LOG_INFO, "%s\n", ifMetricBuf ) ) != 0 )
+        {
+            return (-1);
+         }
+    
+        if( cur -> next != NULL )
         {
             cur = cur -> next;
          }
